/*
******************************************************************************************
*                                                                                        *
* File:	cargill.cc                                                                       *
* Copyright (c) 1994 by Abraxas Software. All rights reserved.                           *                                                                             
* Author:	Shuming Tan                                                              *
* Purpose:	CodeCheck rules developed from the recommendations presented in          *
* 		Tom Cargill's book "C++ Programming Style"                               *
*                                                                                        *
******************************************************************************************
*/

#include <check.cch>

/*
#define NAME_MAX      256
#define VFUNC_MAX     100
#define CLSS_MAX      100
#define DYNAMIC_MAX   200
#define PARAM_MAX      50

char pre_class_name[NAME_MAX],cur_class_name;
char pre_virtual_func_name[VFUNC_MAX][NAME_MAX],cur_virtual_func_name[VFUNC_MAX][NAME_MAX];
int cvfcnt,pvfcnt;
int first_class=1;

struct vtab
{
  char name[NAME_MAX];
  int newcnt,
      delcnt;
} vtab[DYNAMIC_MAX];

int d_pcnt,b_pcnt,b_ccnt;
int d_pbase[PARAM_MAX];
*/

int i,j,k,p;
int new_cnt,
    delete_cnt;

int op_assgn;
int in_a_class;
int in_func;
int forget;

int is_return,
    not_return;
int in_class;





/**************************** Chapter 1.   Abstraction ************************************/


/*
********************************************************************************************
* Principle 1.1: Concentrate common abstractions in a base class.                          *
* If a class is a derived class and some of its base classes are abstract                  *
* classes. Check the declarations of these base classed to see if they have any member     *
* functions(virtual) with same name. If so, consider to concentrate them to the same       *
* base class. The first step is to work with the case that all these involved base         *
* classes themselves are not derived classes.                                              *
* Check all the abstract classes, to see if they have any common member virtual            *
* functions with same name. Because at this stage, the behavior of these functions         *
* has not been defined yet. Consider to put them into a base class to reduce the code      *
* redundancy and increase its maintainability.                                             *
* For each of the found abstract class, find its member functions, and keep a record in    *
* the symbol table. After compare all found functions, mark the functions with same names. *                                                              ********************************************************************************************
*/

/*
********************************************************************************************
* This rule assumes that the complex data structures, struct and array which are disallowed
* in the current version of CodeCheck, are available. So it can not work with CodeCheck yet.
********************************************************************************************
*/

/*
if (mod_begin) first_class=1;

if (tag_begin&&tag_kind==CLASS_TAG) 
{
  pvfcnt=0;
  cvfcnt=0;
}
    

if (dcl_virtual)          //  if a virtual function declaration is encountered, record its name 
{                         //  and the class name in which this function is declared.            
  if (first_class)
  {
    strcpy(pre_class_name,tag_name());
    strcpy(pre_virtual_func_name[pvfcnt++],dcl_name());
  }
  else
  {
    strcpy(cur_class_name,tag_name());
    strcpy(cur_virtual_func_name[cvfcnt++],dcl_name());
  }
}

if (tag_end&&tag_kind==CLASS_TAG)   //at the end of class declaration, compare the virtual 
{                                   //function names with those in previously declared class 
  if (first_class) first_class=0;   //to find if there is any virtual functions with same name.                  
  else
  {
    i=0;
    while(i<pvfcnt) 
    {
      j=0;
      while (j<cvfcnt)
        {
          if (strcmp(pre_virtual_func_name[i],cur_virtual_func_name[j++])==0)
            warn(1001,"class \"%s\" and \"%\" have common virtual function \"%s\".",pre_class_name,cur_class_name,pre_virtual_func_name[i]);
        }
    }
    i=0;                        //  copy the current class and virtual functions names for 
    while (i<cvfcnt)            //  next round of comparison 
    {
       strcpy(pre_virtual_func_name[i],cur_virtual_func_name[i]);
       i++;
    }
    strcpy(pre_class_name,cur_class_name);
    pvfcnt=cvfcnt;
  }      

*/

/*
*****************************************************************************************
* Principle 1.2: A Class should describable a set of objects.                           *
* Once an aggregate type is declared without a data member, the warning message issued. *
*****************************************************************************************
*/

if (tag_kind>ENUM_TAG)         // the enum type is not considered in this case.
  if (tag_members==0) 
    warn(1002,"Class \"%s\" should describe a set of objects",tag_name());


/*
*****************************************************************************************
* Principle 1.3: Use data members for variation in value; reserve virtual functions for *
*                variation in behavior.                                                 *
* For the member function in a derived class which was declared in the base class as    *
* a virtual function, if it only returns a constant value, it is suggested that to use  *
* a data member instead of a virtual function. A member function consumes more resources*
* than a simple data member.                                                            *
*****************************************************************************************
*/

/*
*****************************************************************************************
* This rule assumes that following Codecheck predefined variables which are not in the 
* current version already had existed. So it can not work with CodeCheck yet.
*
* dcl_from_virtual: 1 when a function declared in a derived class has been declared as 
*                   a virtual function in the base class.
* exp_constant:     1 when a constant expression is encountered.
*****************************************************************************************
*/

/*
int is_return,      // set when a return statement is found 
    not_return;     // set when a statement other than return is found 
int is_const;       // set when a constant returned value is found in the return statement 
int in_class;       // set when a class encountered, only functions in a class make sense 
int from_virtual;   // set when a function is declare as virtual in base class, only virtual 
                    // functions make sense 
*/
/*
if (dcl_member)       
  if (dcl_function)
    if (dcl_from_virtual)) 
    {
      in_class=1;
      from_virtual=1;
    }

if (fcn_begin&&from_virtual&&in_class) 
{
  is_return=0;
  not_return=0;
  is_const=0;
}
    

if (lex_keyword)
  if (strcmp(token(),"return")==0)
  {
    is_return=1;
    if (exp_constant) is_const=1;
  }
  else not_return=1;

if (fcn_end&&from_virtual&&in_class)
{
  if (is_return&&!not_return&&is_const) 
    warn(1003,"Use data member for variation in value instead using member function \"%s\".",fcn_name());
  from_virtual=0;
  in_class=0;
}
*/


/*
*************************************************************************************************
* Following is an alternative of the above rule to make it work with CodeCheck.There is a point
* needs to be cleared. It is about the value of return statement in variable stm_kind. As noticed
* that the vaiable stm_kind only be effective to return statement without returned value.
* So here the trigger used to detect a return statement only can be lex_keyword(). And without a
* proper trigger to identify a constant expression, this rule only can detect those member 
* functions with only return statement.
*************************************************************************************************
*/

if (dcl_member)
  if (dcl_function) in_class=1;

if (fcn_begin&&in_class) 
{
  is_return=0;
  not_return=0;
}
    

if (lex_keyword)
  if (strcmp(token(),"return")==0) is_return=1;
  else not_return=1;

if (fcn_end&&in_class)
{
  if (is_return&&!not_return) warn(1003,"Use data member for variation in value instead using member function \"%s\".",fcn_name());
  in_class=0;
}




/***************************************************************************************\
* Principle 1.4: A public derived class should be a specialization of its base class.   *
* This is specially applied to constructors. To do this is to check the parameters of   *
* the constructors in the derived class to see if they match the parameters of          *
* constructors in the base class. Basically, if they do not match, it could be not      *
* following this rule.                                                                  *
\***************************************************************************************/

/*
*****************************************************************************************
* This rule assumes that following predefined variables and functions which are not in
* CodeCheck were available.
* 1. go_to_dcl(char* name), for a use of a type 
*****************************************************************************************
*/

/*
if (tag_base_public) 
  if (dcl_base==CONSTRUCTOR_TYPE) 
  {
    d_pcnt=dcl_parm_count;
    i=0;
    while (i<d_pcnt)
    {
      d_pbase[dcl_parameter]=dcl_base;
      i++;
    }
                
    go_back_base_dcl(tag_name());
    b_ccnt=tag_constructors;
    j=0;
    while (j<b_ccnt)
    {
      if (dcl_base==CONSTRUCTOR_TYPE
      {
        j++;
        b_pcnt=dcl_parm_count;
        if (b_pcnt==d_pcnt)
        {
          i=0;
          while (i<d_pcnt)
          {
            if (d_pbase[i]==dcl_parameter) i++;
            else break;
          }
          if (i==d_pcnt) 
          {
             ok=1;
             break;
          }
        }
      }
    if (j==b_ccnt) warn(1004,"A derived class could be a specialization of its base class");
    go_back_to_prev();
  }
*/
/*
if (mod_begin) cls_cnt=0;

if (tag_base_public)
*/
/***************************************************************************************\
* Principle 1.5: Polymorphism is not the solution to every programming problem.         *
* It depends programmers' experiences, subjective.                                      *
\***************************************************************************************/


/************************* Chapter 2.   Consistency ************************************/


/***************************************************************************************\
* Principle 2.1: A constructor should put its object in well-defined state.             *
* The constructors are used to initialize its objects. At least now we can check if     *
* there is any dynamic data member to be initialized.                                   *
\***************************************************************************************/

if (tag_begin) in_class=1;

if (dcl_variable&&in_class)   
  if (dcl_level(0)==POINTER) 
    warn(2001,"A constructor should put its object in well-defined state.");

if (tag_end) in_class=0;


/**************************************************************************************\
* Principle 2.2: Consider default arguments as an alternative to function overloading. *
* Check the constructors of a class, if this class has more than one constructors and  *
* one of this constructors is default. Suggest that using default arguments as an      *
* alternative to function overloading here.                                            *
\**************************************************************************************/

if (tag_constructors>1&&tag_kind==CLASS_TAG)
  if (tag_has_default) 
    warn(2002,"Consider defaults arguments as an alternative to function overloading in class \"%s\".",tag_name());

/* This rule is based on the compound data structures, like struct, array, which are not 
   supported by the current edition of CodeCheck. So it is not workable yet.           */
  
/*
int i,j,k;
int fcn_cnt;        // count the function number in a tag type
int in_class        
struct fcn_item     // the data structure to record the function name and its arguments
{
  char fcn_name[NAME_MAX];
  int fcn_parm_type[PARM_MAX];
  int fcn_parm_cnt;
} fcn_tbl[FUNC_MAX];

if (tag_begin) 
{
  fcn_cnt=0;
  in_class=1;
}

if (op_open_funargs&&in_class)
  strcpy(fcn_tbl[fcn_cnt].fcn_name,prev_token());

if (dcl_parameter&&in_class)
{
  k=dcl_parameter;
  fcn_tbl[fcn_cnt].fcn_parm_type[k-1]=dcl_base;
}

if (op_close_funargs&&in_class)
{
  fcn_tbl[fcn_cnt].fcn_parm_cnt=k;
  i=0;
  while (i<fcn_cnt)
  {
    if (strcmp(fcn_tbl[i].fcn_name,fcn_tbl[fcn_cnt].name)==0)
      if (fcn_tbl[i].fcn_parm_cnt<fcn_tbl[fcn_cnt].fcn_cnt)
      {
        if (fcn_tbl[i].fcn_parm_cnt==0) 
        {
          warn(2002,"Consider default arguments as an alternative to function overloading for function \"%s\".",fcn_tbl[i].fcn_name);
          break;
        }
        j=0;
        while (j<fcn_tbl[i].fcn_parm_cnt)
        {
          if (fcn_tbl[i].fcn_parm_type[j]!=fcn_tbl[fcn_cnt].fcn_parm_type[j]) 
          {
            no_match=1;
            break;
          }
          j++;
        }
        if (j==fcn_tbl[i].fcn_parm_cnt)
        {
          warn(2002,"Consider default arguments as an alternative to function overloading for function
\"%s\".",fcn_tbl[i].fcn_name);
          break;
        }
     }
     else
     {
       if (fcn_tbl[fcn_cnt]==0)
       {
         warn(2002,"Consider default arguments as an alternative to function overloading for function \"%s"\.",fcn_tbl[fcn_cnt].fcn_name);
         break;
       }
       j=0;
       while (j<fcn_tbl[fcn_cnt].fcn_parm_cnt)
       {
         if (fcn_tbl[i].fcn_parm_type[j]!=fcn_tbl[fcn_cnt].fcn_parm_type[j]) break;
         j++;
       }
       if (j==fcn_tbl[fcn_cnt].fcn_parm_cnt)
       {
         warn(2002,"Consider default arguments as an alternative to function overloading for function \"%s\".",fcn_tbl[fcn_cnt].fcn_name);
         break;
       }
     }
   else i++;
   }
   k=0;
   fcn_cnt++;
}
    
if (tag_end&&in_class) in_class=0;    
*/
  
    
/**************************************************************************************\
* Principle 2.3: Define object states consistently--identify class invariants.         *
* Programmer should bear this in mind. However, there is not any place for a programmer*
* to identify the invariants in C++ syntax. There is no way to use Codecheck rules to  *
* decide if a identifier should be a invariant.                                        *
\**************************************************************************************/

/**************************************************************************************\
* Principle 2.4: Define a class interface consistently--avoid surprises.               *
* Subjective,Just a general guideline.                                                 *
\**************************************************************************************/

/**************************************************************************************\
* Principle 2.5: Identify the delete for every new.                                    *
* For the object instance created with new, destroy it after use by delete to          *
* avoid memory leak.                                                                   *
* The simpler way to do it is just to count the number of the appearances of keyword   *
* NEW and DELETE. If they do not match, there are some potential problems in it.       *
\**************************************************************************************/

/* This principle does not applies to contructors and destructors.                    */

if (dcl_base==CONSTRUCTOR_TYPE||dcl_base==DESTRUCTOR_TYPE) forget=1;

if (fcn_begin)
  if (!forget) { new_cnt=0;delete_cnt=0;in_func=1;}

if (keyword("new")&&in_func) new_cnt++;

if (keyword("delete")&&in_func) delete_cnt++;

if (fcn_end)
  if (!forget)
  {
    if (new_cnt!=delete_cnt)
      warn(2005,"Be aware that identify a DELETE for each NEW in function \"%s\".",fcn_name());
    in_func=0;
    forget=0;
  }


/* Following is a more sophisticated way to implement it. But it is not workable currently. */
/* For each operator NEW or DELETE, record the name of its operand, for each of these variable
   to check if the number of NEW and DELETE does not much, if the number does not mach, probable
   memory leak could happen.                                                                */

/*
if (fcn_begin) 
{ 
  p=0;
  vtab[p].name[0]=0;
  vtab[p].newcnt=0;
  vtab[p].delcnt=0;
  }

if (keyword("new"))
{
   strcpy(name,prev_token(2));
   i=0;
   while (i<p)
   {
     if (strcmp(name,vtab[i].name)!=0) i++; else break;
   }
   if (i==p) 
   {
     strcpy(vtab[p].name,name);
     vtab[p].newcnt++;
     p++;
   }
   else
   {
     vtab[i].newcnt++;
   }
}

if (keyword("delete"))
{
  strcpy(name,next_token(2));
  i=0;
  while (i<p)
  {
    if (strcmp(name,vtab[i].name)!=0) i++;else break;
  }
  if (i==p)
  {
     strcpy(vtab[p].name,name);
    vtab[p].delcnt++;
    p++;
  }
  else
  {
    vtab[i].delcnt++;
  }
}

if (fcn_end)
{
  i=0;
  while (i<p)
  {
    if (vtab[i].newcnt!=vtab[i].delcnt)
      warn(2004,"the number of NEW and DELETE for \"%s\" in \"%s\".",vtab[i].name,fcn_name());
    i++;
  }
}
*/

           
/**************************************************************************************\
* Principle 2.6: When define operator=, remember x=x.                                  *
* X=X could lead some strange behaviors if the operation like DELETE involved.         *
\**************************************************************************************/
/*
Does not work yet, the usage of "tag_has_assign" need to be clarified.
if (tag_has_assign) 
  warn(2006,"Be aware of the unexpected behavior caused by X=X" in class \"%s\".", tag_name());
*/

/*
if (tag_begin&&tag_kind==CLASS_TAG) in_a_class=1;

if (in_a_class&&dcl_function)  
  if (strequiv(dcl_name(),"operator=")) 
    warn(2006,"There is a operator= definition in class %s.",tag_name());

if (tag_end) in_a_class=0;
*/

if (dcl_function)
  if (strcmp(dcl_name(),"operator=")==0) op_assgn=1;

if (fcn_begin&&op_assgn) in_func=1; 

/* the DELETE operation in the OPERATOR= could cause some undefined states */
if (keyword("delete")&&in_func)
  warn(2007,"Possible bizzare behavior caused by X=X in function %s",dcl_name());

if (fcn_end) in_func=0;

 
/**************************************************************************************\
* Principle 2.7: Replace repetitive expressions by calls to a common function.         *
* This is supposed to eliminate the repeated code by placing it in a common            *
* function. To this, we need to check each piece of code to see if it is just          *
* a repetition of some previous piece of code. Doing this is very time-consuming.      *
* And how big is that piece of code should be. From an expression or a compound        *
* statement?                                                                           *
\**************************************************************************************/

/**************************************************************************************\
* Principle 2.8: Avoid computing and storing state information that is never used.     *
\**************************************************************************************/

/*tag_unused is assumed a CodeCheck variable.*/
/*
if (tag_unused!=0) 
  warn(2008,"Avoid computing and storing the state information never used in class %", tag_name());
*/

/*********************** Chapter 3.   Unnecessary Inheritance *************************/

/**************************************************************************************\
* Principle 3.1: Look for simple abstractions.                                         *
* Lack an objective criteria about complexity and simplicity.                          *
\**************************************************************************************/

/**************************************************************************************\
* Principle 3.2: Recognize inheritance for implementation; use a private base class or *
*                (preferably) a member object.                                         *
* Subjective. Mostly it depends on the personal like and the understanding to the      *                                                                       * concepts of inheritance.Though the member object is preferred.                       *       
\**************************************************************************************/

/*
if (tag_base_private)
  warn(3002,"Use a member object preferably instead of a private base class.");
*/

/**************************************************************************************\
* Principle 3.3: Consider default arguments as an alternative to function overloading. *
* Mentioned the previous chapter.                                                      *
\**************************************************************************************/


/************************ Chapter 4.   Virtual Functions ******************************/


/**************************************************************************************\
* Principle 4.1: A derived class must treat inherited state consistently with its      *
*                base class.                                                           *
\**************************************************************************************/


/*
#define MEMB_MAX     100

struct pdmtab
{
  char cname[NAME_MAX];
  char mname[MEMB_MAX][NAME_MAX];
  int pdmcnt;
};

struct pdmtab pdmmbr[CLSS_MAX];
int i,j;
int cls_cnt,                   // counter for classes
    pdm_cnt;                   // counter for protected data members


if (mod_begin) cls_cnt=0;

if (tag_begin&&tag_kind==CLASS_TAG)
{
  pdm_cnt=0;
  if (keyword("protect"))
  {
    strcpy(pdmmbr[cls_cnt],cname,tag_name());
    if (dcl_variable) 
    {
      strcpy(pdmmbr[cls_cnt],mname[pdm_cnt++],dcl_name());
      pdmmbr[cls_cnt].pdmcnt=pdm_cnt;
    }
  }
}
 
if (tag_end&&tag_kind==CLASS_TAG) cls_cnt++;

if (dcl_member)
  if (dcl_function)
    if (fcn_begin)
      if (stm_operands>0)
       if (stm_nonexec)
        if (idn_global)
        {
          i=0;
          while (i<cls_cnt)
          {
             if (strcmp(pdmmbr[i].cname,tag_name())==0) break;
             else
             {
               j=0;
               while (j<pdmmbr[i].pdmcnt)
               {
                 if (strcmp(pdmmbr[i].mname,opn_name())==0)
                 {
                   warn(4001,"Potential inconsistency");
                   break;
                 }
               j++;
             }
           }
           i++;
         }
       } 
                          
            
*/ 

/**************************************************************************************\
* Principle 4.2: If a public base class does not have a virtual destructor, no derived *
*                class nor members of a derived class should have a destructor.        *
\**************************************************************************************/

/*
****************************************************************************************
* The following code is assumed that CodeCheck has the backward chaining mechanism.
* the next piece of code is write without this assumed mechanism.
****************************************************************************************
*/
/*
if (tag_pub_derived)
{
  if (dcl_base==DESTRUCTOR_TYPE)
  {
    back_to_dcl(tag_name());
    if (dcl_base==DESTRUCTOR_TYPE&&dcl_virtual) 
    {
      back_to_prev();
      warn(4002,"Do not define a destructor in class %s.",tag_name()); 
    }
  }
}
*/

/*
****************************************************************************************
* The idea here is straight. When a class is encountered, check if it has a virtual 
* destructor, then put the class name into the symbol table. Afterwards, if a derived
* class is detected, then check if it has a destructor. If so, look through the symbol
* table for the name of its base class name. If the name not there, this means the
* derived class should not have a destructor.
* The used compound data structures are array which has not been in the current
*  edition of CodeCheck. So this rule is not workable currently 
****************************************************************************************
*/

/*
int vd_cnt;                           // Virtual destructor counter
char vd_tbl[CLSS_MAX][NAME_MAX];      // Symbol table for virtual destructor's class
char cname[NAME_MAX];                 // class name

if (mod_begin) vd_cnt=0;

if (tag_begin)                        // Where a possible virtual destructor declared
  if (dcl_base==DESTRUCTOR_TYPE&&dcl_virtual)
  {
    strcpy(vd_tbl[vd_cnt].name,tag_name());  // Put the class name containing a virtual destructor
    vd_cnt++;
  }

if (tag_base_public)        // Check if a derived class has a destructor and its base class does 
{                           // not have a virtual destructor.
  strcpy(cname,base_name());
  if (tag_begin)
    if (dcl_base==DESTRUCTOR)
    {
      i=0;
      while(i<vd_cnt)
      {
        if (strcmp(vd_tbl[i],cname)==0)
        {
          warn(4002,"Class \"%s\" should not have a destructor.");
          break;
        }
        i++;
      }
    }
}

if (mod_end) vd_cnt=0;  
*/


/**************************************************************************************\
* Principle 4.3: Usually, The destructor in a public base class should be virtual.     *
* Once a public derived class detected, go to the base class declaration. If the       *
* destructor is not declared virtual, issue warning message.                           *
\**************************************************************************************/

/*
if (mod_begin) vd_cnt=0;

if (tag_begin)
{
  if (dcl_base==DESTRUCTOR&&dcl_virtual)
  {
    strcpy(vd_tbl[vd_cnt],tag_name());
    vd_cnt++;
  }
}

if (tag_base_public)
{
  strcpy(cname,base_name());
  i=0;
  while(i<vd_cnt)
  {
    if (strcmpy(cname,vd_tbl[i]) i++
    else break;
  }
  if (i==vd_cnt)
    warn(4003,"The destructor in class \"%s\" should be virtual.",cname);
}

if (tag_end) vd_cnt=0;
*/


/**************************************************************************************\
* Principle 4.4: Migrate common behavior to the base class.                            *
* Just as what done in first chapter, rule 1.                                          *
\**************************************************************************************/


/**************************************************************************************\
* Principle 4.5: Reduce coupling--minimize interactions between classes.               *
* It is supposed that the member function, if it is not a friend function of other     *
* class, access the variables not declared in the class in which it is a member of.    *
\**************************************************************************************/

/* any function in a class should not change the variables defined in other classes */
/*
int in_non_friend_member_func;

if (dcl_member)
  if (dcl_function)
    if (!dcl_friend) in_non_friend_member_func=1;

if (fcn_begin&&in_non_friend_member_func)
  if (idn_global)                          
    warn(4005,"Probable coupling with other class on var \"%s\" in function \"%s\".",idn_name(),fcn_name());

if (fcn_end&&in_non_friend_member_func) in_non_friend_member_func=0;
*/
      

/**************************************************************************************\
* Principle 4.6: Use data members for variation in value;reserve virtual functions for *
*                variation in behavior.                                                *
* Mentioned in previous chapter.                                                       *
\**************************************************************************************/


/**************************************************************************************\
* Principle 4.7: No class is perfect; too narrow a design is better than broad.        *
* Subjective                                                                           *
\**************************************************************************************/


/**************************************************************************************\
* Principle 4.8: Consider default arguments as an alternative to function overloading. *
* Mentioned in previous chapter.                                                       *
\**************************************************************************************/

/************************* Chapter 5.   Operator Overloading **************************/

/**************************************************************************************\
* Principle 5.1: Write clearly--do not be too clever.                                  *
* General guideline. Subjective.                                                       *
\**************************************************************************************/


/**************************************************************************************\
* Principle 5.2: The meaning of an overloaded operator should be natural, not clever.  *
* This depends on programmer's intention what this operator will do.                   *
* For example, the overladed operator^ is intended to get the power of a number.       *
* Syntactically, it is legal to do so. However, the natural meaning in C/C++ of        *
* operator is bitwise exclusive-or. To implement this principle, the rule needs to     *
* understand what is exact meaning of this operator by looking at the function         *
* body. It is difficult to do.                                                         *
\**************************************************************************************/


/**************************************************************************************\
* Principle 5.3: Overload operators consistently.                                      *
\**************************************************************************************/


/**************************************************************************************\
* Principle 5.4: A set of overloaded operators should be complete.                     *
* Mentioned in previous chapter.                                                       *
\**************************************************************************************/


/**************************************************************************************\
* Principle 5.5: When defining operator=, remember x=x;                                *
* This has been mentioned in previous chapter.                                         *
\**************************************************************************************/

/**************************************************************************************\
* Principle 5.6: When overloading operators, avoid surprises.                          *
* Subjective.It is about the semantic meaning of the overloading operators.            *                                          
\**************************************************************************************/


/**************************************************************************************\
* Principle 5.7: Recognize inheritance for implementation: use a private base class or *
*                (preferably) a member object.                                          *
* Mentioned the previous chapter.                                                      *
\**************************************************************************************/


/**************************************************************************************\
* Principle 5.8: an overloaded operator must interact appropriately                    *
*                with other operators.                                                 *
\**************************************************************************************/


/****************************** Chapter 6.   Wrappers *********************************/

/* The wrapper discussed here is the C++ class used as a software layer around 
   a utility library written in C to improve the interface of these library functions.*/


/**************************************************************************************\
* Principle 6.1: Know the valid lifetime of a pointer return from a function.          *
\**************************************************************************************/

/*
if (tag_begin)
  if (fcn_begin)
    if (op_call&&fcn_is_dcl_extern_link)
      if (this_fcn_caller_returns_a_pointer)
        warn(6001,"Beware of the life time of the pointer.");
*/
/*
int xlf_cnt;
char xlf_tbl[XLFCN_MAX][NAME_MAX];

if (mod_begin) xlf_cnt=0;

if (dcl_extern_linkage)
  if (dcl_function)
    if (dcl_levels>0)
      if (dcl_level(0)==POINTER)
        strcpy(xlf_tbl[xlf_cnt++],dcl_name());

if (dcl_member)
  if (fcn_begin)
    if (op_call)
    {
      i=0;
      while (i<xlf_cnt)
      {
        if (strcmp(xlf_tbl[i],op_function())==0)
        {
          warn(6001,"Be aware of the life time of the pointer got from function \"%s\".",op_function());
          break;
        }
        i++;
      }
    }

if (mod_end) xlf_cnt=0;
*/

/**************************************************************************************\
* Principle 6.2: Independent objects should have independent behavior.                 *
\**************************************************************************************/


/**************************************************************************************\
* Principle 6.3: Do not encapsulate essential information                              *
*                --make it available by some means.                                    *
* What information should be not encapsulated? It depends the behavior of the          *
* individual program. Subjective.                                                      *	
\**************************************************************************************/


/**************************************************************************************\
* Principle 6.4: The behavior of an object that has encountered an error               *
*                should be well-defined.                                               *
\**************************************************************************************/


/**************************************************************************************\
* Principle 6.5: Make a C++ wrapper an improvement over the C interface.               *
* Subjective. It is too broad to the improvement by a C++ wrapper. It is the goal      *                                                                      * to make C++ wrapper a better interface than C interface.                             *
\**************************************************************************************/



/**************************** Chapter 7.   Efficiency *********************************/



/**************************************************************************************\
* Principle 7.1: Reduce coupling--minimize interactions between classes.               *
* Mentioned in previous chapter.                                                       *
\**************************************************************************************/


/**************************************************************************************\
* Principle 7.2: Don't guess. Use an execution profile to isolate performance problems.*
* It looks not in the coverage of CodeCheck yet.                                       *
\**************************************************************************************/


/**************************************************************************************\
* Principle 7.3: Look in class implementations for the source of problems.             *
* This is more about the semantics of the programming, not syntax. And it is too       *
* broad.                                                                               *
\**************************************************************************************/


/**************************************************************************************\
* Principle 7.4: Look in client code for the source of performance problems.           *
\**************************************************************************************/


/**************************************************************************************\
* Principle 7.5: A complete interface invites efficient client code.                   *
\**************************************************************************************/


/**************************************************************************************\
* Principle 7.6: Look for simple abstractions.                                         *
* Mentioned before.                                                                    *
\**************************************************************************************/



/***************************** Chapter 8.   A Case Study ******************************/


/**************************************************************************************\
* Principle 8.1: Remember the null byte--use new char[strlen(s)+1].                    *
\**************************************************************************************/


/*
if (op_call&&strcmp(op_function(),"strcpy")==0) 
  warn(8001,"remember the null byte at the end of string");
*/

/* This could be extended to the problem of string overwriting regarding to the
   functions of string opreations like strcpy(), sprintf(), etc.                      */

/* 
if (fcn_begin) 
{
  assgn=0;
  crt_strng=0;
  rltv_len=0;
  last_look=0;
  pr_cnt=0;
}

if (op_assign) 
  if (op_base(1)==CHAR_TYPE&&op_level(1,0)==POINTER)
  {
    strcpy(dest_name,prev_token());
    assgn_to_ptr=1;
  }

if (op_new&&assgn_to_ptr)
  if (op_base(1)==CHAR_TYPE&&op_level(1,0)==POINTER) crt_strng=1;

if (op_open_subcript&&crt_strng)
  if (op_call&&strcmp(op_function(),"strlen")==0) rltv_len=1;

if (op_open_funargs&& rltv_len)
{
  strcpy(src_name,next_token());
  last_look=1;
}

if (op_add&&last_look)
  if (next_token()=="1") 
  {
    strcpy(pair[pr_cnt].d_name,dest_name);
    strcpy(pair[pr_cnt].s_name,src_name);
    pr_cnt++;
    assgn_to_ptr=0;
    crt_strng=0;
    rltv_len=0;
    last_look=0;
  }

if (op_call&&op_function()=="strcpy")
{
  strcpy(dest_name,op_name(1));
  strcpy(src_name,op_name(2));
  i=0;
  while (i<pr_cnt)
  {
    if (pair[i].d_name==dest_name&&pair[i].s_name==src_name) break;
    i++;
  }
  if (i==pr_cnt)
    warn(8001,"******");
}

if (fcn_end) 
{
  pr_cnt=0;
  assgn_to_ptr=0;
  crt_strng=0;
  rltv_len=0;
  last_look=0;
}
  
*/
/**************************************************************************************\
* Principle 8.2: Don't use a constructor to initialize static data members.            *
\**************************************************************************************/

/*
if (dcl_base==CONSTRUCTOR_TYPE)
  if (fcn_begin)
    if (a_var_is_changed&&it_is_a_static_member)
      warn(8002,"Do not use a constructor to initialize static data member.");
*/

/*
int smcnt;
int i;
char smtab[MEMB_MAX][NAME_MAX];
char var[NAME_MAX];

if (tag_begin) 
{ 
  smcnt=0;
  if (dcl_member&&dcl_static)
  {
    strcpy(smtab[smcnt],dcl_name);
    smcnt++;
  }
  if (dcl_base==CONSTRUCTOR_TYPE)
  {
    if (fcn_begin)
      if (op_assign)
      {
        strcpy(var,prev_token());
        i=0;
        while(i<smcnt)
        {
          if (strcmp(smtab[i],var)==0)
          {
            warn(8002,"Do not use a constructor to initialize static member %s",var);
            break;
          }
          i++;
        }
      }
  }
}
*/
                 
/**************************************************************************************\
* Principle 8.3: Reduce coupling--minimize interactions between classes.               *
* Mentioned before.                                                                    *
\**************************************************************************************/

/**************************************************************************************\
* Principle 8.4: Each class should serve a single, coherent purpose.                   *
\**************************************************************************************/

/**************************************************************************************\
* Principle 8.5: Design abstract data types rather than modules.                       *
\**************************************************************************************/

/**************************************************************************************\
* Principle 8.6: Use data members for variation in value;reserve virtual functions for * 
*                variation in behavior.                                                *
* Mentioned in previous chapters.                                                      *
\**************************************************************************************/

/**************************************************************************************\
* Principle 8.7: Don't solve a specific case if the general case is simple.            *
\**************************************************************************************/

/**************************************************************************************\
* Principle 8.8: Usually, the destructor in a public base class should be virtual.     *
* Mentioned before.                                                                    *
\**************************************************************************************/


/************************** Chapter 9.   Multiple Inheritance *************************/

/**************************************************************************************\
* Principle 9.1: Before using virtual base classes, understand them thoroughly.        *
* Subjective.This is just a general guideline.                                         *
\**************************************************************************************/

/**************************************************************************************\
* Principle 9.2: Do not try learn the semantics of multiple inheritance                *
*                from your compiler.                                                   *
* Subjective. It suggests that many compilers can not treat inheritance properly       *                                                                          
\**************************************************************************************/

/**************************************************************************************\
* Principle 9.3: Avoid artificially limiting a class to serve only as a base class.    * 
* Subjective. This is just a guideline for practice in using inheritance.              *
\**************************************************************************************/

/**************************************************************************************\
* Principle 9.4: If a public base class does not have a virtual destructor,            *
*                no derived class should have a virtual constructor.                   *
* Mentioned in previous chapters.                                                      *
\**************************************************************************************/

/**************************************************************************************\
* Principle 9.5: If a multiple inheritance hierarchy has any destructors,              *
*                every base class should have a virtual destructor.                    *
\**************************************************************************************/

/* assume that following CodeCheck variables already existed:
   tag_base_public: set to 1 when a public base detected in a tag definition and parsing
                    position is after the base class name.
   tag_base_protected: set to 1 when a protected base detected in a tag definition.
   tag_base_private: set to 1 when a private base detected in a tag definition.        */

/*
int vd_cnt,
    bt_cnt;
char vd_tbl[VDEST_MAX][NAME_MAX];
char tb_tbl[BASE_MAX][NAME_MAX];

if (mod_begin) vd_cnt=0;

if (tag_base_public||tag_base_protected||tag_base_private)
  strcpy(bt_tbl[bt_cnt++],prev_token());

if (tag_begin)
{
  if (dcl_base==DESTRUCTOR_TYPE)
    if (bt_cnt>0)
    {
      i=0;
      while (i<bt_cnt)
      {
        j=0;
        while (j<vd_cnt)
        {
          if (strcmp(vd_tbl[j],bt_tbl[i])==0) break;
          j++;
        }
        if (j==vd_cnt) 
        {
          warn(9005,"Class \"%s\" should have a virtual destructor.",bt_tbl[i]);
        }
        i++;
     }
   }  
   else
   {
     if (dcl_virtual) strcpy(vd_tbl[vd_cnt++],tag_name());
   }
}  
  
if (tag_end) bt_cnt=0;

if (mod_end) vd_cnt=0;  
*/ 

/*    
/**************************************************************************************\
* Principle 9.6: Use a member object, not inheritance, to include a server             *
*                in client object.                                                     *
* It has the same meaning with rule                                                    *
\**************************************************************************************/


/**************************************************************************************\
* Principle 9.7: A class should describe a set of objects.                             *
* Mentioned in previous chapters.                                                      *
\**************************************************************************************/
