// This rule checks that if a class declaration has keyword
// class begins from first column of the line
// for example, 
// class C
// {
//   ...
// };
// Keyword class is at the beginning of the line.


if (keyword("class")) //find keyword class
{
   // check if keyword class is in template parameter
   if (!prev_token())
   {
     if (strcmp(prev_token(),"<")!=0||strcmp(prev_token(),",")!=0)
     {
       if (strstr(line(),"class")!=line())
       {
         warn(1,"put class at the beginning of the line,");
       }
     }
   }
   else
   {
     if (strstr(line(),"class")!=line())
     {
       warn(1,"put class at beginning of the line.");
     }
   }
}

// This rule checks if a class's three sections(public, protected and
// private) appear exactly in that order. 
// However, it is assumed that there is no nested classes to make
// this rule work properly.

int public_member_cnt;    // count public members
int protected_member_cnt; // count protected members
int private_member_cnt;   // count private members

if (dcl_member==3&&lin_within_class) // exclude the member decl outside of
{                                    // the class body.
  if (dcl_access==0) // is public member
  {
    public_member_cnt++;
    // only first public member will get this message
    // to reduce the redundant messages
    if (private_member_cnt&&public_member_cnt==1)
    {
      warn(2,"private section found before public section.");
    }
    if (protected_member_cnt&&public_member_cnt==1)
    {
      warn(3,"protected section found before public section.");
    }
  }
  if (dcl_access==1) //is protected member
  {
    protected_member_cnt++;
    {
      if (private_member_cnt&&protected_member_cnt==1)
      {
        warn(4,"private section found before protected section.");
      }
    }
  }
  if (dcl_access==2) //is private member
  {
    private_member_cnt++;
  }
}

// This rule checks if all class members are indented with 3 
// spaces. And no tab character is used for indentation.
if (tag_end)
{
  // reset variables for next class.
  public_member_cnt=0;
  protected_member_cnt=0;
  private_member_cnt=0;
}

if (dcl_member&&lin_within_class)
{
  if (lin_indent_tab!=0)
  {
    warn(5,"tab character used in indentation");
  }
  else
  {
    if (lin_indent_space!=3)
    {
      warn(6,"class member is not indented by 3 spaces.");
    }
  }
}

if (lin_end)
{
  if (lin_indent_tab!=0)
  {
    warn(5,"Tab character used in indentation.");
  }
}   

