/*
######################################################################
#                                                                    #
#  FILE NAME : cscps.cc                                              #
#  BY        : Shuming Tan                                           #
#  Copyright (c) 1994 by Abraxas Software. All rights reserved.      #
#  PURPOSE   :  Rules Developed following                            #
#               Credintrans Specific C Programming Standard          #
#                                                                    #
######################################################################
*/

#include <check.cch>
int i;

/************* A - Specific use of basic functions ******************/

/*
 * A01: The value returned by strcpy() has no signification;
 *      Do not use it.
 */

/*
 * A02: The sign of strcmp() returned value has no signification;
 *      You can only compare this value with zero.
 */

/*
 * A03: Do not use strlen() on a string exceeding 32767 characters.
 */ 

/*
 * A04: Only use toupper() on lower case letters.
 */
int toupper_call;

if (idn_function)
{
  toupper_call=0;
  if (strcmp(idn_name(),"toupper")==0)
    toupper_call=1;
}

if (lex_constant&&toupper_call)
{
  if (strlen(token())!=1||(token())[0]<'a'||(token())[0]>'z')
    warn(1004,"Only use lower case char as parameter of function toupper()");
  toupper_call=0;
}


/*
 * A05: Do not use the fopen() function.
 */

if (idn_function)
  if ( strcmp(idn_name(),"fopen")==0)
    warn(1005,"Do not use the fopen() function.");

/*
 * A06: The third parameter of fseek() can not be the SEEK_END macro.
 */

int macro_found;

if (macro("SEEK_END")) macro_found=1;

if (idn_function)
  if (!strcmp(idn_name(),"fseek")&&macro_found)
  {
     warn(1004,"Do not use SEEK_END as parameter of fseek().");
     macro_found=0;
  }
  
/*
 * A07: The third parameter of itoa() can only be 10.
 */

if (op_call)
  if (strcmp(op_function(),"itoa")==0)
    if (strcmp(prev_token(),"10")!=0)
      warn(1007,"The third parameter of itoa() can only be 10");

/*
 * A08: The only characters authorized within the sprintf() format
 *      (behind the %) are d for decimal, c for single character, 
 *      and s for chharacter string.
 */
  
/*
 * A09: Same rule as A2, but more difficult to check, because of the 
 *      use of ret as an intermediate variable.
 */

/**************** B - Programming Constraints *******************/

/*
 * B01: Do not cast a long constant; Use the L suffix.
 */

int is_long, is_long_const;

if (keyword("long")) is_long=1;

if (lex_constant)
 if (is_long)
 {
   is_long_const=1;
   is_long=0;
 }

if (op_cast)
  if (is_long_const)
  {
    warn(2001,"Do not cast a long constant, use L suffix.");
    is_long_const=0;
  }

/*
 * B02: Use constants within these limits:
 *      unsigned char           0...255   (1 byte )
 *      int              -32767...32767   (2 bytes)
 *      long    -214783647...2147483647   (4 bytes)
 */

/*
 * Any constant can be detected. However, what we can get now are only
 * tokens of these constants. Unless the new functions added to convert
 * strings to numbers.
 */

/*
 * B03: Do not declare an array exceeding 1K.
 */
 

/*
 * B04: Do not use the signed char type specifier.
 */

if (dcl_signed)
  if (dcl_base==CHAR_TYPE)
    warn(2004,"Do not use signed char type specifier on %s.",dcl_name());

/*
 * B05: Do not use the register storage class specifier.
 */

if (dcl_storage_flags&REGISTER_SC)
  warn(2005,"Do not use the register storage class specifier on %s.", dcl_name());


/*
 * B06: Do not use volatile type specifier.
 */

if (dcl_variable||dcl_function)
{
  i=0;
  while (i<=dcl_levels)
  {
    if (dcl_level_flags(i)&VOLATILE_FLAG)
      warn(2006,"Do not use volatile type specifier on %s.",dcl_name());
    i=i+1;
  }
}
/*
 * B07: You have to write the type specifier.
 */

if (dcl_no_specifier)
  warn(2007,"Do not omit the type specifier for %s.",dcl_name());

/*
 * B08: Do not use ?: operator.
 */

if (op_cond)
  warn(2008,"Do not use ?: operator.");

/*
 * B09: A function parameter can not be a pointer to function.
 */

if (dcl_parameter)
  if (dcl_level(1)==FUNCTION&&dcl_level(0)==POINTER)
    warn(2009,"A function parameter can not be a pointer to function.");
  
/*
 * B10: The type specifier of a function can not be short.
 */

if (dcl_function)
  if (dcl_base==SHORT_TYPE)
    warn(2010,"The type specifier of a function(%s) can not be short.",dcl_name());

/*
 * B11: You have to write a function declaration like this,
 *      using the PROTOTYPE macro,
 *      static void function name PROTOTYPE (long, char *);
 */

int prototype;

if (macro("PROTOTYPE")) prototype=1;

if (dcl_function)
  if (!dcl_definition)
  {
    if (!prototype) 
      warn(2011,"Use macro PROTOTYPE in function prototype.");
    prototype=0;
  }
/*
 * B12: Do not modify a constant string.
 */

if (op_assign)
{
  if (op_level(2,0)==POINTER)
     if (op_level_flags(2,0)&CONST_FLAG)
        warn(2012,"Do not modify aconstant string.");
}

/*
 * B13: You can not modify a function parameter which
 *      could be a constant string.
 */

/*
 * B14: Do not use or compare a char with negative values.
 */

/*
 * There is no way to recognize any constant which is negative 
 */

/* 
 * B15: The index of a struct array has to be int.
 */
 
int is_struct_array;

if (idn_variable)
  if (idn_base==STRUCT_TYPE&&idn_level(0)==ARRAY)
    if (strcmp(token(),"[")==0) is_struct_array=1;

if (op_subscript)
{
  if (is_struct_array)
    if (op_base(1)!=INT_TYPE)
       warn(2015,"The index of struct array has to be int.");
    is_struct_array=0;
}


/*
 * B16: Do not initialize auto arrays, or structs, or unions.
 */

if (dcl_storage_flags&AUTO_SC)
  if (dcl_initializer)
  {
    if (dcl_level(0)==ARRAY||dcl_base==STRUCT_TYPE||dcl_base==UNION_TYPE)
      warn(2016,"Do not initialize auto array, struct or union %s.",dcl_name());
  }

/*
 * B17: You can not declare const and no const values within the same struct.
 */

if (dcl_storage_flags&CONST_FLAG)
  if (!dcl_initializer)
    warn(2017,"Do not declare constant %s without value.",dcl_name());

/*
 * B18: An assignment operator has to be isolated.
 */


/*
 * B19: You have to do a cast to compare or assign a pointer to
 *      a pointer to a different type.
 */

if (cnv_ptr_to_ptr)
  warn(2019,"Do not assign a pointer to another pointer without cast.");

/*
 * B20: You can not assign a pointer to something else
 *      than another pointer.
 */

if (op_assign)
{
   if(op_level(1,0)==POINTER&&op_level(2,0)!=POINTER)
     warn(2020,"Do not assign a pointer to something else than other pointer.");
}

/*
 * B21: A pointer can not address a prohibited memory place, even
 *      if this address is not read or written.
 */

/*
 * B22: Do not use sizeof on an array like this:
 *      1. the size of this array ia not given explicitly,
 *         but calculated at the initialization.
 *      2. this array is not referenced in the source excepted
 *         at the definition, the declaration, or a sizeof().
 */
int dim,j,l;

if (op_sizeof)
{
  dim=1;
  l=op_levels(1);
  j=0;
  while (j<=l)
  {
     if (op_level(1,j)==ARRAY) 
     {
        dim=dim*op_array_dim(1,j);
     }
     j++;
  }
  if (dim<0) warn(2022,"Do not use sizeof on the array no fixed size.");
}
       

/*
 * B23: You have to initialize an external variable when you
 *      declare it ( even a partial initialization ).
 */

if (dcl_variable)
  if (dcl_global)
    if (!dcl_extern)
      if (!dcl_initializer) 
        warn(2023,"Initialize the declared external variable %s.",dcl_name());

/*
 * B24: You can not use a pointer to void.
 */

if (dcl_base==VOID_TYPE)
  if (dcl_level(dcl_levels-1)==POINTER)
     warn(2024,"Do not use a pointer to void on %s.",dcl_name());

/*
 * B25: You can not use & operator between two char variables.
 */

if (op_bit_and)
  if (op_base(1)==CHAR_TYPE&&op_base(2)==CHAR_TYPE)
    warn(2025,"Do not use & operator between two char variables.");

/*
 * B26: Expression behind switch has to be int or char.
 */

if (op_switch)
  if (op_base(1)!=INT_TYPE&&op_base(1)!=CHAR_TYPE)
    warn(2026,"Expression behind switch is not int or char.");

/*
 * B27: Expression behind a case can not be casted.
 */

int is_case;
int case_with_cast;

if (keyword("case"))
  is_case=1;

if (op_cast)
  if (is_case) case_with_cast=1;


if (op_colon_1)
  if (case_with_cast)
  {
    warn(2027,"Expression behind a case can not be casted.");
    is_case=0;
    case_with_cast=0;
  }


/******************* C - Style Recommendations *****************/

/*
 * C01: Indent is three blanks.
 */

if (lin_indent_space!=3) 
  warn(3001,"Indentation of line %d is not 3 blanks.",lin_number);

/*
 * C02: #if is at column 1.
 */

if (pp_white_before)
  warn(3002,"Do not put blanks before proprecessor directives.");

/*
 * C03: The opening brace is placed at the end of its statement line;
 *      The closing brace is placed on the same column as the first
 *      character of this statement.
 */

/*
 * C04: An identifier contains letters and digits only;
 *      A macro name can contain '_' (underscore).
 */

if (dcl_global||dcl_local)
  if (strstr(dcl_name(),"_")||strstr(dcl_name(),"$"))
    warn(3004,"Identifier %s contains non letter-digit character.",dcl_name());

if (pp_macro)
  if (strstr(pp_name(),"$"))
    warn(3004,"Macro %s contains non-letter-digit-underscore char.",pp_name());

/*
 * C05: A macro name can not exceed 31 characters.
 */

if (pp_macro>31)
  warn(3005,"Macro name %s longer than 31 characters.",pp_name());

/*
 * C06: Every identifier has to be written with a specific form.
 *      For instance, a union typedef will be named ***xxxU.
 */

/*
 * C07: The name of a macro must begin with MAC.
 */

if (pp_macro)
  if (strstr(pp_name(),"MAC")!=pp_name())
    warn(3007,"The name of macro %s must begin with MAC.",pp_name());

/*
 * C08: PUSH, POP, TRUE, FALSE, and other names...,
 *      can not be used as identifiers.
 */

if (dcl_global||dcl_local)
{
  if (strcmp(dcl_name(),"PUSH")==0)
    warn(3008,"Do not use PUSH as identifier.");
  if (strcmp(dcl_name(),"POP")==0)
    warn(3008,"Do not use POP as identifier.");
  if (strcmp(dcl_name(),"TRUE")==0)
    warn(3008,"Do not use TRUE as identifier.");
  if (strcmp(dcl_name(),"FALSE")==0)
    warn(3008,"Do not use FALSE as identifeir.");
}

/*
 * C09: You must write a specific comment before every function definition.
 */

if (fcn_no_header)
  warn(3009,"Do not forget write a specific comment before function %s.",fcn_name());

/*
 * C10: You must write a comment after every struct member.
 */

int has_member_dcl;

if (dcl_member==2) has_member_dcl=1;

if (lin_end)
{
  if (!lin_has_comment&&has_member_dcl)
    warn(3010,"Do not forget to write a comment after each struct member.");
  has_member_dcl=0;
}

/*
 * C11: You can not initialize an extern variable when you declare it.
 */

if (dcl_extern)
  if (dcl_initializer)
    warn(3011,"Do not initialize extern variable %s when declared.",dcl_name());

/*
 * C12: A definition can not be specified extern.
 */

if (dcl_definition)
  if (dcl_extern)
     warn(3012,"A definition can not be specified extern.");

/*
 * C13: An external function has to be declared and defined extern.
 */

/*
 * C14: You can use struct or union labels only for recursive
 *      definitions or crossed references.
 */
 
/*
 * C15: A function body can not exceed 30 lines.
 */
 
if (fcn_total_lines>30)
  warn(3015,"The body of function %s can not exceed 30 lines.",fcn_name());

/*
 * C16: A source file name can not exceed 7 characters.
 */

if (mod_begin)
  if (strlen(mod_name())>7)
    warn(3016,"Source file name %s can not exceed 7 chars.",mod_name());

/*
 * C17: Every project file has to be written with a specific form.
 *      For instance, a C file with external definitions will be
 *      named ***ext.c.
 */

/*
 * C18: A constant string can not contain an opening comment.
 */

if (lex_constant==5)
   if (strstr(token(),"/*"))
     warn(3018,"Do not contain opening comment in string constant %s.",token());

/*
 * C19: Within a struct or array initialization, the comma between 
 *      the last value and the closing brace is prohibited.
 */
int has_init;

if (dcl_base==STRUCT_TYPE||dcl_level(0)==ARRAY)
  if (strcmp(token(),"=")==0) has_init=1;

if (op_close_brace)
  if (has_init)
    if (strcmp(prev_token(),",")==0)
    {
      warn(3019,"Do not put comma before closing brace in array or struct initializer.");
      has_init=0;
    }

/*
 * C20: You have to use PNUL macros as null pointer, do not use NULL nor 0.
 */


/******************** D - Project Rules **********************/

/*
 * D01: An extern defined variable has to be used within another source file.
 */

/*
 * D02: You have to initialize a static variable before using it.
 */

if (dcl_static) 
  if (dcl_variable)
    if (!dcl_initializer)
      warn(4002,"Initlialize static variable %s before using it.",dcl_name());

/*
 * D03: A const declared variable has to be defined const, and vice versa.
 */

/*
 * D04: Always use prototyped function declarations.
 */

if (dcl_function)
  if (dcl_no_prototype)
    warn(4004,"Use prototyped function declaration for %s", dcl_name());

/*
 * D05: A static identifier can not be named as an extern identifier.
 */

/*
 * D06: Names are not case sensitive.
 */

/*
 * D07: You have to use & operator to address the first element in an array.
 */

/*
 * D08: To initialize a char* struct or union member with a constant
 *      string, you have to use the CASTPCHAR macro.
 */

