/*	Copyright (c) 1988-94 by Abraxas Software.
 *
 *	This is the start of a collection of CodeCheck
 *	rules for testing for general portability.
 */


if ( keyword("near") || keyword("far") || keyword("huge")
		|| keyword("pascal") || keyword("fortran")
		|| keyword("cdecl")  || keyword("interrupt") )
	warn( 1001, "Not a portable keyword." );

if ( identifier("entry") )
	warn( 1002, "Not a portable identifier." );	

if ( lex_assembler )
	warn( 1003, "Embedded assembler." );

if ( lex_backslash )
	warn( 1004, "Nonportable backslash." );

if ( lex_char_empty )
	warn( 1006, "The empty character constant is not portable." );

if ( lex_char_long )
	warn( 1007, "Multi-character \'char\' values are not portable." );

if ( lex_float )
	warn( 1008, "The suffix F is not portable." );

if ( lex_radix == 16 )
	warn( 1009, "Hex constants are not portable." );

if ( lex_long_float )
	warn( 1010, "Illegal in ANSI C." );
	
if ( lex_nl_eof )
	warn( 1011, "ANSI source files must end with a newline character." );	

if ( lex_nonstandard )
	warn( 1012, "Nonstandard character." );

if ( lex_num_escape )
	warn( 1011, "Numeric escape sequences may not be portable." );

if ( lex_str_length > 509 )
	warn( 1012, "This string may be too long for portability." );

if ( lex_unsigned )
	warn( 1013, "The suffix U is not portable." );

if ( lex_wide )
	warn( 1014, "The prefix L is not portable." );

if ( pp_ansi )
	warn( 1015, "Portable only among ANSI compilers." );

if ( pp_bad_white )
	warn( 1016, "This whitespace character is not portable." );

if ( pp_comment )
	warn( 1017, "This use of whitespace is not portable." );

if ( pp_depend )
	{
	warn( 1018, "Another macro depends on this macro." );
	warn( 1019, "Result of this #undef is NOT portable!" );
	}

if ( pp_if_depth > 3 )
	warn( 1020, "#if nesting > 3 may not be portable." );

if ( pp_include_depth > 5 )
	warn( 1021, "Include depth may not be portable." );

if ( pp_length > 80 )
	warn( 1022, "Macro definition too long for portability." );

if ( pp_pragma )
	warn( 1023, "Pragmas are generally not portable." );

if ( pp_recursive )
	warn( 1024, "Recursive macro definitions are not portable." );

if ( pp_sizeof )
	warn( 1025, "Very few compilers will accept this use of sizeof." );

if ( pp_sub_keyword )
	warn( 1026, "This #define is not legal in most compilers." );

if ( pp_unstack )
	warn( 1027, "Unstacking macro definitions is not portable." );

if ( pp_trailer )
	warn( 1028, "Trailing tokens in this directive are not portable." );

if ( pp_unknown )
	warn( 1029, "Not a portable preprocessor directive." );

if ( pp_white_before || pp_white_after )
	warn( 1030, "This use of whitespace is not generally portable." );

if ( op_based )
	warn( 1031, "This operator is not portable." );

if ( dcl_extern_ambig )
	warn( 1032, "This name agrees with another on first 6 characters." );

if ( dcl_auto_init )
	warn( 1033, "Initializing automatic variables is not portable." );

if ( dcl_initializer )
	if ( dcl_extern )
		warn( 1034, "Initializing extern variables is not portable." );

if ( dcl_typedef_dup )
	warn( 1035, "Duplicated type definitions are not portable." );

if ( dcl_underscore )
	warn( 1036, "This variable name may not be portable." );

if ( dcl_union_init )
	warn( 1037, "Union initializers are not generally portable." );

if ( lin_length > 80 )
	warn( 1038, "Lines longer than 80 characters may not be portable." );

if ( lex_nested_comment )
	warn( 1039, "Nested comments are not portable." );

if ( lex_cpp_comment )
	warn( 1040, "// comments are not portable." );

if ( stm_goto )
	warn( 1041, "Block initializations may fail on goto here." );

if ( mod_macros > 500 )
	warn( 1042, "Too many macros for some compilers." );

if ( pp_include_white )
	warn( 1043, "Nonportable leading whitespace in filename." );

if ( lex_enum_comma )	
	warn( 1044, "Comma ending enum list is not portable." );

if ( dcl_typedef_dup )	
	warn( 1045, "Repeated definition of type %s.", dcl_name() );

if ( lex_intrinsic )
	warn( 1046, "Intrinsic function %s is not portable.", token() );

if ( exp_not_ansi )
	warn( 1047, "This expression is not permitted in ANSI C." );

if ( pp_paste_failed )
	warn( 1048, "Paste operation failed, result is undefined." );

if ( pp_not_ansi )
	warn( 1049, "Not portable to strict ANSI preprocessors." );
