
//   These CodeCheck rules illustrate one way of enforcing
//   a standard order upon the elements of a C source file. 

//   Copyright (c) 1990-93 by Abraxas Software. All rights reserved.
//   Author: Loren Cobb, Abraxas Software.

//   This file detects violations of the following order:

//   1.  Header file #include directives.
//   2.  External variable declarations.
//   3.  Function declarations.
//   4.  Macro definitions.
//   5.  Type definitions.
//   6.  Public variable definitions (external linkage).
//   7.  Private variable definitions (static identifiers).
//   8.  Function definitions.


///////////////////////////////
//   Module Initialization:  //
///////////////////////////////

int  stage;          //  Module format stage (range: 0-8)

if ( mod_begin )
	stage = 0;       //  Reinitialize stage at start of every module.


/////////////////////////////
//   Module Format Rules:  //
/////////////////////////////

if ( pp_include )
	{
	if ( (stage > 1) && (stage != 4) )
		warn( 1001, "This #include of file %s is out of sequence.", header_name() );
	stage = 1;
	}

if ( dcl_global )
	if ( lin_source )
		{
		if ( dcl_function )
			{
			if ( dcl_definition )
				stage = 8;
			else
				{
				if ( stage > 3 )
					warn( 1003, "Declaration of function %s is out of sequence.", dcl_name() );
				stage = 3;
				}
			}
		else if ( dcl_extern )
			{
			if ( stage > 2 )
				warn( 1002, "External declaration of %s is out of sequence.", dcl_name() );
			stage = 2;
			}
		else if ( dcl_static )
			{
			if ( stage > 7 )
				warn( 1007, "Definition of private variable %s is out of sequence.", dcl_name() );
			stage = 7;
			}
		else
			{
			if ( stage > 6 )
				warn( 1006, "Definition of public variable %s is out of sequence.", dcl_name() );
			stage = 6;
			}
		}

if ( pp_macro )
	if ( lin_source )
		{
		if ( stage > 4 )
			warn( 1004, "This macro definition is out of sequence." );
		stage = 4;
		}

if ( dcl_typedef )
	if ( lin_source )
		{
		if ( stage > 5 )
			warn( 1005, "Definition of typename %s is out of sequence.", dcl_name() );
		stage = 5;
		}
