//	Copyright (c) 1988-93 by Abraxas Software. All rights reserved.

//	These CodeCheck rules are based on the C coding standards
//	developed by a major East coast software firm.

//  This file is used by company programmers to monitor
//  their compliance with corporate standards.

//  The warnings are intended to be seen in the context of
//  a list file. Use:  check myproject.ccp -Rsample -L

//  Warning codes:  1000  lexical
//                  2000  keyword
//                  3000  preprocessor
//                  4000  declaration
//                  5000  general

//  These rules may serve as a basis for customizing your
//  own set of company standards for C code production.



#include <check.cch>

/**********  Lexical Rules  **********/

if ( lex_nonstandard )
	warn( 1001, "Nonstandard character." );

if ( lex_unsigned || lex_float )
	warn( 1002, "Do not use this suffix." );

if ( (lex_radix == 8) || (lex_radix == 16) )
	warn( 1003, "Do not use octal or hex constants." );

if ( lex_str_length > 509 )
	warn( 1005, "String literal too long for ANSI C." );

if ( lex_str_trigraph )
	warn( 1006, "ANSI C will recode the trigraph in this string." );

if ( lex_nested_comment )
	warn( 1007, "Do not nest comments." );

if ( op_low )
	{
	if ( ! op_white_before )
		warn( 1008, "Put space before operator %s.", token() );
	if ( ! op_white_after )
		warn( 1008, "Put space before operator %s.", token() );
	}

/**********  Keyword rules  **********/

if ( keyword("int") )
	if ( ! lex_macro_token )
		warn( 2001, "Use INTEGER, short, or long." );

if ( keyword("register") )
	if ( ! lex_macro_token )
		warn( 2002, "Use the REGISTER macro here." );

if ( keyword("volatile") )
	warn( 2003, "The \'volatile\' keyword is not portable." );


/**********  Preprocessor Rules  **********/

if ( pp_sub_keyword )
	warn( 3001, "Do not substitute preprocessor keywords." );

if ( pp_stack || pp_benign )
	warn( 3002, "Do not redefine macros without an #undef." );

if ( pp_include & 1 )
	warn( 3003, "#include macros are not allowed." );

if ( lex_not_manifest )
	warn( 3004, "Define this constant in a macro." );

if ( pp_arg_paren )
	warn( 3005, "Surround this argument with parentheses." );

if ( pp_comment )
	warn( 3006, "Surround this comment with space." );

if ( pp_white_before )
	warn( 3007, "The # must be in column 1 for portability." );

if ( pp_trailer )
	warn( 3008, "Place these tokens in a comment." );

if ( pp_if_depth > 8 )
	warn( 3009, "#if nesting > 3 is not be portable." );

if ( pp_include_depth > 8 )
	warn( 3010, "Include depth exceeds 8." );

if ( pp_pragma )
	warn( 3011, "Pragmas are generally not portable." );

if ( pp_arg_multiple )
	warn( 3012, "Possible undesired side-effects may occur here." );

if ( pp_assign )
	warn( 3013, "The \"=\" sign in this macro may be an error." );

if ( pp_keyword )
	warn( 3014, "Warning: this macro redefines a keyword." );

if ( pp_overload )
	warn( 3015, "This identifier conflicts with a macro function name." ); 

if ( pp_semicolon )
	warn( 3016, "Macro body ends with a semicolon." );

if ( pp_undef )
	warn( 3017, "Use #undef as seldom as possible." );

if ( pp_unstack )
	warn( 3018, "Undefining multiply-defined macros is not portable." );


/**********  Declaration Rules  **********/

if ( dcl_no_specifier )
	{
	if ( dcl_function )
		warn( 4001, "Explicit function return type required." );
	else
		warn( 4002, "Explicit type specifier required." );
	}

if ( dcl_bitfield_anon )
	warn( 4003, "All bitfields must have names." );

if ( dcl_union_init )
	warn( 4004, "Do not initialize unions." );

if ( dcl_auto_init )
	warn( 4005, "Do not initialize auto structs or arrays." );

if ( dcl_union_bits )
	warn( 4006, "Do not use bitfields in unions." );

if ( dcl_tag_def )
	if ( lin_source && (dcl_base == STRUCT_TYPE) )
		warn( 4007, "Define structures in header files only." );

if ( dcl_oldstyle )
	warn( 4008, "ALWAYS use prototypes." );

if ( idn_no_prototype )
	warn( 4009, "Function %s needs a prototype.", idn_name() );

if ( dcl_need_3dots )
	warn( 4010, "ANSI C requires 3 dots (...) here." );

if ( dcl_hidden )
	warn( 4011, "This declaration hides another." );


/**********  General Rules  **********/

if ( macro("offsetof") )
	warn( 5001, "Do not use offsets!" );

