/*
 *    Copyright (c) 1988-93 by Abraxas Software.
 *
 *	This is a collection of CodeCheck rules for
 *	measuring function, module, and project size.
 *
 *  NOTE: The CodeCheck line- and token-counting variables count
 *  only source-file lines and tokens. To include lines and
 *  tokens from header files, use the command-line option -F.
 *
 */


int   mod_dcl_lines,   //  module  declaration lines
      mod_ppd_lines,   //  module  preprocessor directive lines
      mod_sup_lines,   //  module  lines suppressed by preprocessor
      mod_pun_lines,   //  module punctuation lines
      prj_dcl_lines,   //  project declaration lines
      prj_ppd_lines,   //  project preprocessor directive lines
      prj_sup_lines,   //  project lines suppressed by preprocessor
      prj_pun_lines,   //  project punctuation lines
      count_headers,   //  1 if should count header lines, else 0
	  banner_done;     //  1 if banner has been printed, else 0

int		unknown;



if ( mod_begin )
	{
	if ( ! banner_done )
		{
		printf( "\nDate: %s.\n", time_stamp() );
		count_headers = option('F');  // Count header lines if option -F is set by user
		if ( count_headers )
			{
			printf( "Note: Line counts include lines from header files." );
			set_option( 'S', 1 );     // Make sure rules are evaluated in headers!
			}
		else
			printf( "Note: Line counts do NOT include lines from header files." );
		banner_done = 1;
		}
	printf( "\n\n------------------- %s.c -------------------\n\n", mod_name() );
	reset( fcn_exec_lines );
	mod_dcl_lines = 0;
	mod_ppd_lines = 0;
	mod_sup_lines = 0;
	mod_pun_lines = 0;
	}

if ( lin_end )
	if ( count_headers || lin_source )
		if ( ! (lin_is_exec || lin_is_comment || lin_is_white) )
			{
			if ( lin_preprocessor )
				++mod_ppd_lines;
			else if ( lin_suppressed )
				++mod_sup_lines;
			else if ( lin_dcl_count )
				++mod_dcl_lines;
			else
                                ++mod_pun_lines;
			}

if ( mod_end )
	{
	prj_dcl_lines += mod_dcl_lines;
	prj_ppd_lines += mod_ppd_lines;
	prj_sup_lines += mod_sup_lines;
	prj_pun_lines += mod_pun_lines;
	
	printf( "Executable lines:        %5d", mod_exec_lines );
	printf( "      Low-level statements:       %5d\n", mod_low );
	printf( "Comment lines with text: %5d", mod_com_lines );
	printf( "      Non-executable statements:  %5d\n", mod_nonexec );
	printf( "Declaration lines:       %5d", mod_dcl_lines );
	printf( "      High-level statements:      %5d\n", mod_high );
	printf( "Whitespace lines:        %5d", mod_white_lines );
	printf( "          TOTAL STATEMENTS........%5d\n", mod_low + mod_high + mod_nonexec );
	printf( "Preprocessor lines:      %5d\n", mod_ppd_lines );
	printf( "Suppressed lines:        %5d", mod_sup_lines );
	printf( "      Macros defined:             %5d\n", mod_macros );
	printf( "Punctuation lines:       %5d", mod_pun_lines );
	printf( "      Functions defined:          %5d\n", mod_functions );
	printf( "    TOTAL LINES..........%5d\n\n", mod_total_lines );

	if ( ncases(fcn_exec_lines) > 0 )
		{
		printf( "Executable lines per function:\n" );
		printf( "  Mean:    %6.2f",  mean(fcn_exec_lines) );
		printf( "    Minimum: %4.0f\n",   minimum(fcn_exec_lines) );
		printf( "  Std.Dev: %6.2f",  stdev(fcn_exec_lines) );
		printf( "    Maximum: %4.0f\n\n", maximum(fcn_exec_lines) );
		}
	}

if ( prj_end )
	if ( prj_modules > 1 )
		{
		printf( "\n---------------- PROJECT ----------------\n\n" );
		printf( "Executable lines:        %6d", prj_exec_lines );
		printf( "      Low-level statements        %5d\n", prj_low );
		printf( "Comment lines with text: %6d", prj_com_lines );
		printf( "      Non-executable statements:  %5d\n", prj_nonexec );
		printf( "Declaration lines:       %6d", prj_dcl_lines );
		printf( "      High-level statements:      %5d\n", prj_high );
		printf( "Whitespace lines:        %6d", prj_white_lines );
		printf( "          TOTAL STATEMENTS........%5d\n", prj_low + prj_high + prj_nonexec );
		printf( "Preprocessor lines:      %6d", prj_ppd_lines );
		printf( "      Macros defined:             %5d\n", prj_macros );
		printf( "Suppressed lines:        %6d", prj_sup_lines );
		printf( "      Functions defined:          %5d\n", prj_functions );
		printf( "Punctuation lines:       %6d", prj_pun_lines );
		printf( "      Number of modules:          %5d\n", prj_modules );
		printf( "    TOTAL LINES..........%6d", prj_total_lines );
		printf( "      Number of headers:          %5d\n\n", prj_headers );
		
		printf( "\n================== END ==================\n" );
		}
